#pragma once

#include <iosfwd>
#include <vector>

#include "vec3f.h"

namespace geometry {

class Polyline3f {
public:
  using point = math::Vec3f;
  using points_store = std::vector<point>;
  using iterator = points_store::iterator;
  using const_iterator = points_store::const_iterator;

public:
  Polyline3f() = default;
  Polyline3f(points_store points);

  point operator[](int idx) const;
  point &operator[](int idx);
  point front() const;
  point back() const;

  void setPoint(int idx, point p);
  void addMidpointToSegment(int idx);
  void removePoint(int idx);

  size_t pointCount() const;
  point const *data() const;
  points_store const &points() const;

  iterator begin();
  iterator end();

  const_iterator begin() const;
  const_iterator end() const;

private:
  points_store m_points;
};

// Free functions
float length(Polyline3f const &curve);

std::vector<math::Vec3f>
midpointSubdivide(std::vector<math::Vec3f> const &points);

Polyline3f midpointSubdivide(Polyline3f const &polyline);

} // namespace geometry
