#include "polyline3f.h"

namespace geometry {

using point = Polyline3f::point;
using points_store = Polyline3f::points_store;

Polyline3f::Polyline3f(points_store points) : m_points(points) {}

point Polyline3f::operator[](int idx) const { return m_points[idx]; }

point &Polyline3f::operator[](int idx) { return m_points[idx]; }

point Polyline3f::front() const { return m_points.front(); }

point Polyline3f::back() const { return m_points.back(); }

void Polyline3f::setPoint(int idx, point point) { m_points.at(idx) = point; }

Polyline3f::iterator Polyline3f::begin() { return m_points.begin(); }

Polyline3f::iterator Polyline3f::end() { return m_points.end(); }

Polyline3f::const_iterator Polyline3f::begin() const {
  return m_points.begin();
}

Polyline3f::const_iterator Polyline3f::end() const { return m_points.end(); }

void Polyline3f::addMidpointToSegment(int idx) {
  auto a = m_points[idx];
  size_t neighbourIdx = idx + 1;
  neighbourIdx = neighbourIdx >= m_points.size() ? 0 : neighbourIdx;
  auto b = m_points[neighbourIdx];

  auto c = lerp(a, b, 0.5f);
  // TODO: std::advance wasn't working?
  m_points.insert(std::begin(m_points) + neighbourIdx, c);
}

void Polyline3f::removePoint(int idx) {
  m_points.erase(std::begin(m_points) + idx);
}

size_t Polyline3f::pointCount() const { return m_points.size(); }

point const *Polyline3f::data() const { return m_points.data(); }

points_store const &Polyline3f::points() const { return m_points; }

// Free functions
float length(Polyline3f const &curve) {
  float l = 0.f;
  int numLineSegments = curve.pointCount() - 1;

  for (int i = 0; i < numLineSegments; ++i) {
    l += distance(curve[i], curve[i + 1]);
  }
  return l;
}

std::vector<math::Vec3f>
midpointSubdivide(std::vector<math::Vec3f> const &points) {
  points_store newPoints;
  newPoints.reserve(points.size() * 2);

  int numLineSegments = points.size() - 1;

  for (int i = 0; i < numLineSegments; ++i) {
    auto mid = lerp(points[i], points[i + 1], 0.5f);
    newPoints.push_back(points[i]);
    newPoints.push_back(mid);
  }

  newPoints.push_back(points.back());

  return newPoints;
}

Polyline3f midpointSubdivide(Polyline3f const &polyline) {
  return Polyline3f(midpointSubdivide(polyline.points()));
}

} // namespace geometry
