#include <fstream>  // read/write to files
#include <iostream> // read/write to console
#include <string>   // typical text strings
#include <vector>   // default storage you should use in most cases

#include <algorithm> // helpful algorithms in the STL library
#include <numeric>

#include "polyline3f.h"
#include "vec3f.h"

using namespace std;

namespace math {
math::Vec3f mean(std::vector<math::Vec3f> const &values) {
  auto accum = Vec3f();

  if (values.empty())
    return accum;

  for (auto const &value : values) {
    accum += value;
  }

  return accum / values.size();
}

bool writePointsToFile(std::vector<math::Vec3f> const &points,
                       std::string const &filePath) {
  ofstream file(filePath);
  if (!file)
    return false;

  for (auto const &point : points) {
    file << point << '\n';
  }
  // file.close(); // implicitly called when leaving scope of function
  return true;
}

} // namespace math

int main() {

  math::Vec3f x{1, 0, 0}, y{0, 1, 0};

  cout << endl;
  cout << "x: " << x << '\n';
  cout << "y: " << y << '\n';


  auto a = x;
	auto b = math::Vec3f(0,1,0);
  math::Vec3f c(a);

  cout << endl;
  cout << "(a + b): " << (a + b) << '\n';
  cout << "2.f * (a + b): " << 2.f * (a + b) << '\n';
  cout << "(c * b): " << (c * b) << '\n';

  cout << endl;
  cout << "x cross y: " << (x ^ y) << '\n';
	cout << "y cross x: " << math::cross(y, x) << '\n';

  using math::Vec3f;
  Vec3f d = a - b;

  cout << endl;
  cout << "(a-b): " << d << '\n';
  cout << "(-a): " << -a << '\n';

  a.normalize();     // insitu
  b = normalized(b); // return value

  int const pointsCount = 4;
  std::vector<math::Vec3f> points;
  // points.resize(pointsCount);
  points.reserve(pointsCount);

  for (int i = 0; i < pointsCount; ++i) {
    points.push_back(math::Vec3f(i, 0, 1));
  }

  cout << "\nPOINTS\n";
  for (auto const &p : points) {
    cout << p << '\n';
  }

  cout << "\nMEAN VALUE " << math::mean(points) << '\n';

  writePointsToFile(points, "points.txt");

  geometry::Polyline3f curve(points);
  cout << "\nPolyline length (pre subdivision): " << length(curve) << '\n';
  curve = midpointSubdivide(curve);
  // curve = midpointSubdivide(curve);
  cout << "\nPOINTS\n";
  for (auto const &p : curve) {
    cout << p << '\n';
  }
  cout << "\nPolyline length (post subdivision): " << length(curve) << '\n';

  writePointsToFile(curve.points(), "curve.txt");

  return 0;
}
